# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/datasetsDynamic',
                'doc_host': 'https://kaiguender.github.io',
                'git_url': 'https://github.com/kaiguender/datasetsDynamic',
                'lib_path': 'datasetsDynamic'},
  'syms': { 'datasetsDynamic.core': {'datasetsDynamic.core.foo': ('core.html#foo', 'datasetsDynamic/core.py')},
            'datasetsDynamic.loadData': {'datasetsDynamic.loadData.loadDataYaz': ('yaz.html#loaddatayaz', 'datasetsDynamic/loadData.py')},
            'datasetsDynamic.loadDataBakery': { 'datasetsDynamic.loadDataBakery.loadDataBakery': ( 'bakery.html#loaddatabakery',
                                                                                                   'datasetsDynamic/loadDataBakery.py')},
            'datasetsDynamic.loadDataYaz': { 'datasetsDynamic.loadDataYaz.loadDataYaz': ( 'yaz.html#loaddatayaz',
                                                                                          'datasetsDynamic/loadDataYaz.py')},
            'datasetsDynamic.utils': { 'datasetsDynamic.utils.add_lag_features': ( 'utils.html#add_lag_features',
                                                                                   'datasetsDynamic/utils.py'),
                                       'datasetsDynamic.utils.getDayIndex': ('utils.html#getdayindex', 'datasetsDynamic/utils.py'),
                                       'datasetsDynamic.utils.getMonthFeature': ('utils.html#getmonthfeature', 'datasetsDynamic/utils.py'),
                                       'datasetsDynamic.utils.getWeekdayFeature': ( 'utils.html#getweekdayfeature',
                                                                                    'datasetsDynamic/utils.py')}}}
